/**
 * \file graphe-2.h
 * \brief Représentation des graphes par leur matrice d'adjacence
 * \version 1
 * \date lundi 30 septembre 2019, 10:04:11 (UTC+0200)
 * \authors Pierre Rousselin, Antoine Rozenknop, Sophie Toulouse

 * Dans cette version :
 * - L'implémentation de la matrice d'adjacence est
 * réalisée par un tableau *dynamique*, de taille \f$n^2\f$,
 * où \f$n\f$ est l'ordre du graphe.
 * - Les graphes sont pour l'instant tous non orientés.
 *
 * Règles de gestion de la mémoire :
 * 1. Déclarer *une variable* de type graphe.
 * 2. ALLOUER la matrice d'adjacence de ce graphe avec une (et une seule) fonction qui
 * ALLOUE et initialise cette matrice.
 * 3. DÉTRUIRE la matrice d'adjacence après utilisation avec la fonction
 * \a graphe_detruire.
 * Il est alors possible d'ALLOUER
 * de nouveau la matrice d'adjacence en utilisant la même variable.
 *
 * Exemple d'utilisation d'une variable de type graphe :\n
 * \code{.c}
 * graphe g;
 * int statut;
 * statut = graphe_stable(&g, 82);
 * if (statut < 0)
 * 	exit(EXIT_FAILURE);
 * graphe_ajouter_arete(&g, 1, 56);
 * graphe_ecrire_dot(&g, "mon_graphe.dot");
 * graphe_detruire(&g);
 * statut = graphe_complet(&g, 7);
 * if (statut < 0)
 * 	exit(EXIT_FAILURE);
 * graphe_afficher(&g);
 * graphe_detruire(&g);
 * \endcode
 * 
*/

#ifndef GRAPHE_H
#define GRAPHE_H

/* ______________________________________ Structure de données */

/**
 * \brief Graphe non orienté représenté par sa matrice d'adjacence.
 *
 * Un graphe est ici représenté par son nombre de sommets \a n
 * et sa matrice d'adjacence (tableau dynamique de n*n entiers) \a adj.
 *
 * L'ensemble des sommets du graphe est alors \f$\{0, 1, 2, \ldots, n-1\}\f$.
 *
 * Le champ \a m est le nombre d'arêtes et est mis à jour par les fonctions de
 * manipulation.
 */
struct s_graphe {
	int n;/**< nombre de sommets du graphes (doit être >=0) */
	int m;/**< nombre d'arêtes du graphes */
	int *adj;
	/**< matrice d'adjacence du graphe
	 *
	 * Doit être cohérente avec m.
	 *
	* Nombre d'arete de u vers v : adj[v + u * n]
	 */
};
typedef struct s_graphe graphe;

/* ______________________________________ Constructeur et destructeur */

/**
 * \brief ALLOUE de la mémoire pour la matrice d'adjacence d'un graphe d'ordre
 * \a n, et l'initialise en un graphe sans arête
 * \param g adresse d'une variable de type graphe *existante*
 * \param n ordre du graphe
 * \return -1 si l'allocation de mémoire pour la matrice d'adjacence a échoué,
 * 	-2 si \a n est négatif, 0 sinon.
*/
int graphe_stable(graphe* g, int n);

/**
 * \brief Libère la zone mémoire occupée par la matrice d'adjacence d'un graphe
 * g.
 * \param g pointeur vers un graphe dont la matrice d'adjacence est allouée
 *
 * Si g->adj est le pointeur NULL, ne fait rien.
 */
void graphe_detruire(graphe *g);

/* ______________________________________ Ajout / Suppression d'arêtes */

/**
 * \brief Ajoute une arête entre deux sommets dans le graphe
 * \param g adresse de la variable graphe à modifier
 * \param v une extrémité de l'arête (entre 0 et n - 1)
 * \param w une extrémité de l'arête (entre 0 et n - 1)
 */
void graphe_ajouter_arete(graphe* g, int v, int w);

/**
 * \brief Supprime une arête entre deux sommets dans le graphe
 * \param g adresse de la variable graphe à modifier
 * \param v une extrémité de l'arête (entre 0 et n - 1)
 * \param w une extrémité de l'arête (entre 0 et n - 1)
 * \return -1 si l'arête ne peut être supprimée (n'existe pas), 0 sinon
 */
int graphe_supprimer_arete(graphe* g, int v, int w);

/* ______________________________________ Accesseurs en lecture */

/**
 * \brief Retourne l'ordre du graphe.
 * \param g adresse de la variable graphe à lire
 * \return le nombre de sommets du graphe
 */
int graphe_get_n(graphe* g);

/**
 * \brief Retourne le nombre d'arêtes du graphe.
 * \param g adresse de la variable graphe à lire
 * \return le nombre d'arêtes du graphe
 */
int graphe_get_m(graphe* g);

/**
 * \brief Retourne le nombre de fois qu'une arete est présente entre deux
 * sommets donnés.
 * \param g adresse de la variable graphe à lire
 * \param v une extrémité de l'arête (entre 0 et n - 1)
 * \param w une extrémité de l'arête (entre 0 et n - 1)
 * \return le nombre d'arêtes entre les sommets \a v et \a w
 */
int graphe_get_multiplicite_arete(graphe* g, int v, int w);

/* NE PAS ACCÉDER DIRECTEMENT AUX CHAMPS EN-DESSOUS DE CETTE LIGNE */

/**
 * \brief Retourne le degré d'un sommet.
 * \param g adresse de la variable graphe à lire
 * \param v sommet du graphe (entre 0 et n - 1)
 * \return le degré du sommet v dans le graphe
 */
int graphe_get_degre(graphe* g, int v);

/* ______________________________________ Entrées / Sorties */

/**
 * \brief Affiche sur la sortie standard les données d'un graphe
 * \param g adresse de la variable graphe à lire
 *
 * Affiche le nombre de sommets, d'arêtes, la matrice d'adjacence et les degrés
 * des sommets du graphe
 */
void graphe_afficher(graphe* g);

/**
 * \brief ALLOUE de la mémoire pour la matrice d'adjacence d'un graphe d'ordre
 * \a n, et l'initialise en un graphe simple complet
 * \param g adresse d'une variable de type graphe *existante*
 * \param n ordre du graphe
 * \return -1 si l'allocation de mémoire pour la matrice d'adjacence a échoué,
 * 	-2 si \a n est négatif, 0 sinon.
*/
int graphe_complet(graphe* g, int n);

/**
 * \brief ALLOUE de la mémoire pour la matrice d'adjacence d'un graphe d'ordre
 * \a n, et l'initialise en un graphe simple aléatoire d'ordre \a n, où chaque arête
 * est, de façon indépendante, présente avec probabilité \a p.
 * \param g adresse d'une variable de type graphe *existante*
 * \param n ordre du graphe
 * \param p probabilité de présence de chaque arête
 * \return -1 si l'allocation de mémoire pour la matrice d'adjacence a échoué,
 * 	-2 si \a n est négatif, -3 si \a p n'est pas compris entre 0 et 1 et
 * 	0 sinon.
*/
int graphe_aleatoire(graphe* g, int n, double p);

/**
 * \brief Écrit le graphe au format dot dans un fichier.
 * \param g adresse de la variable graphe à lire
 * \param nom_fichier nom du fichier à écrire
 * \return 0 si l'écriture du fichier a réussi, -1 sinon
 */
int graphe_ecrire_dot(graphe *g, char *nom_fichier);

#endif
